/* $Id: cmd_put.c,v 1.3 2003/05/02 18:30:44 short Exp $
 * client cmdline interface command "put" for libcaptive
 * Copyright (C) 2003 Jan Kratochvil <project-captive@jankratochvil.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; exactly version 2 of June 1991 is required
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "config.h"

#include <glib/gmessages.h>
#include <glib/gerror.h>
#include <popt.h>
#include <captive/client-file.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include "cmd_put.h"	/* self */
#include "cmd_cd.h"	/* for cmdline_path_from_cwd() */
#include "main.h"


/* Config: */
#define TRANSFER_BUFFER_SIZE (0x10000)


GQuark cmdline_cmd_put_error_quark(void)
{
GQuark r=0;

	if (!r)
		r=g_quark_from_static_string("cmdline-cmd-put");

	return r;
}


const struct poptOption cmd_put_table[]={
		POPT_AUTOHELP
		POPT_TABLEEND
		};


void cmd_put(const char **cmd_argv,GError **errp)
{
CaptiveFileObject *captive_file_object;
const gchar *sourcefile,*targetfile;
guint perm=0644;
int fdsrc;
guint8 transfer_buffer[TRANSFER_BUFFER_SIZE];
int got;

	g_return_if_fail(!errp || !*errp);

	sourcefile=cmd_argv[0];
	if (cmd_argv[1])
		targetfile=cmdline_path_from_cwd(cmd_argv[1]);
	else {
gchar *sourcefile_basename;

		sourcefile_basename=g_path_get_basename(sourcefile);
		targetfile=cmdline_path_from_cwd(sourcefile_basename);
		g_free(sourcefile_basename);
		}

	if (-1==(fdsrc=open(sourcefile,O_RDONLY
#ifdef O_BINARY
			| O_BINARY
#endif /* O_BINARY */
#ifdef O_LARGEFILE
			| O_LARGEFILE
#endif /* O_LARGEFILE */
			))) {
		g_set_error(errp,CMDLINE_CMD_PUT_ERROR,CMDLINE_CMD_PUT_ERROR_OPENING_SOURCE_FILE,
				_("Error opening source host-os file '%s': %s"),sourcefile,g_strerror(errno));
		return;
		}

	if (!errvfsresult_to_gerr(errp,captive_file_new_create(
			&captive_file_object,	/* captive_file_object_return */
			cmdline_captive_vfs_object,	/* captive_vfs_object */
			targetfile,	/* pathname */
			GNOME_VFS_OPEN_WRITE,	/* mode */
			FALSE,	/* exclusive */
			perm))) {	/* perm */
		err_cleanup(errp);
		g_set_error(errp,CMDLINE_CMD_PUT_ERROR,CMDLINE_CMD_PUT_ERROR_CANNOT_CREATE_TARGET_GUESTOS_FILE,
				_("Cannot create target guest-os file: %s"),targetfile);
		goto err_close_fdsrc;
		}

	while (0<(got=read(fdsrc,transfer_buffer,sizeof(transfer_buffer)))) {
GnomeVFSFileSize bytes_written;

		if (!errvfsresult_to_gerr(errp,captive_file_write(
				captive_file_object,	/* captive_file_object */
				transfer_buffer,	/* buffer */
				got,	/* num_bytes */
				&bytes_written))) {	/* bytes_written_return */
			err_cleanup(errp);
			g_set_error(errp,CMDLINE_CMD_PUT_ERROR,CMDLINE_CMD_PUT_ERROR_WRITING_TARGET_GUESTOS_FILE,
					_("Error writing target guest-os file: %s"),targetfile);
			goto err_unref_captive_file_object;
			}
		if (bytes_written!=(GnomeVFSFileSize)got) {
			g_set_error(errp,CMDLINE_CMD_PUT_ERROR,CMDLINE_CMD_PUT_ERROR_WRITING_TARGET_GUESTOS_FILE,
					_("Error writing target guest-os file '%s': requested %d, written %Lu"),
					targetfile,got,(unsigned long long)bytes_written);
			goto err_unref_captive_file_object;
			}
		}
	if (got==-1) {
		g_set_error(errp,CMDLINE_CMD_PUT_ERROR,CMDLINE_CMD_PUT_ERROR_READING_SOURCE_FILE,
				_("Error reading source host-os file '%s': %s"),sourcefile,g_strerror(errno));
		goto err_unref_captive_file_object;
		}
	g_assert(got==0);

err_unref_captive_file_object:
	g_object_unref(captive_file_object);
err_close_fdsrc:
	if (close(fdsrc)) {
		err_cleanup(errp);	/* may be clean */
		g_set_error(errp,CMDLINE_CMD_PUT_ERROR,CMDLINE_CMD_PUT_ERROR_CLOSING_SOURCE_FILE,
				_("Error closing source host-os file '%s': %s"),sourcefile,g_strerror(errno));
		}
}
